@@ -28,10 +28,16 @@ from elisa.plugins.flickr.resource_provider import API_SERVER
 
 from elisa.plugins.base.models.image import ImageModel
 from elisa.core.media_uri import MediaUri
+from elisa.core.application import CONFIG_DIR
 
 from twisted.internet import defer
 
-import hashlib
+try:
+    from hashlib import md5
+except ImportError:
+    # hashlib is new in Python 2.5
+    from md5 import md5
+
 from xml.dom import minidom
 import os
 
@@ -55,7 +61,7 @@ def sign_arguments(arguments):
     """
     signature = _secret + \
                 ''.join([arg + arguments[arg] for arg in sorted(arguments)])
-    arguments['api_sig'] = hashlib.md5(signature).hexdigest()
+    arguments['api_sig'] = md5(signature).hexdigest()
 
 def _get_token_file():
     """
@@ -64,7 +70,7 @@ def _get_token_file():
     @return: the filename where to store the authentication token
     @rtype:  C{str}
     """
-    return os.path.expanduser(os.path.join('~', '.flickr', _key, 'auth.xml'))
+    return os.path.join(CONFIG_DIR, 'flickr', 'auth.xml')
 
 def _read_token(response):
     """
