@@ -23,6 +23,7 @@ Provide access to resources served by Flickr over HTTP.
 from elisa.core.components.resource_provider import ResourceProvider
 from elisa.core.resource_manager import NoMatchingResourceProvider
 from elisa.core.media_uri import MediaUri
+from elisa.core import common
 
 from elisa.plugins.http_client.http_client import ElisaHttpClient, \
                                                   ElisaAdvancedHttpClient
@@ -30,24 +31,28 @@ from elisa.plugins.http_client.extern.client_http import ClientRequest
 from twisted.web2 import responsecode
 from twisted.web2.stream import BufferedStream, MemoryStream, FileStream
 
-from twisted.internet import defer
+from twisted.internet import defer, threads
 from elisa.core.utils.cancellable_defer import CancellableDeferred, \
                                                CancelledError
 
-
 from elisa.plugins.base.models.media import RawDataModel
 from elisa.plugins.base.models.image import ImageModel
 from elisa.plugins.flickr.models import FlickrResponseModel, \
-                                        FlickrPhotoModel, FlickrTagModel
+                                        FlickrPhotoModel, FlickrTagModel, \
+                                        LoginModel, FlickrPhotoSetModel, \
+                                        FlickrContactModel
 
 import re
 import mimetools
 import os.path
 from xml.dom import minidom
 
+from twill import commands as twill
+
 API_SERVER = 'api.flickr.com'
 IMG_SERVER = 'static.flickr.com'
 
+
 class FlickrResourceProvider(ResourceProvider):
 
     """
@@ -75,7 +80,21 @@ class FlickrResourceProvider(ResourceProvider):
         self._api_client.pipeline_queue = [] # Emulate pipelining
         # One client for each farm
         self._img_clients = {}
-        return defer.succeed(self)
+
+        self.auth_token = None
+
+        def got_error(failure):
+            # we want to ignore the Exception at this point. User can try a new
+            # login later on via the UI anyway.
+            path = common.application.log_failure(failure)
+            self.warning("Login to Flickr failed. Full output at %s" % path)
+            failure.trap(ValueError)
+            return self
+
+        dfr = self.login()
+        dfr.addCallback(lambda r: defer.succeed(self))
+        dfr.addErrback(got_error)
+        return dfr
 
     def clean(self):
         """
@@ -86,13 +105,100 @@ class FlickrResourceProvider(ResourceProvider):
         close_dfrs.append(self._api_client.close())
         # Close all the image farm connections
         close_dfrs += [client.close() for client in self._img_clients.values()]
-        return defer.DeferredList(close_dfrs, consumeErrors=True)
+        dfr = defer.DeferredList(close_dfrs, consumeErrors=True)
+
+        def parent_clean(result):
+            return super(FlickrResourceProvider, self).clean()
+
+        dfr.addCallback(parent_clean)
+        return dfr
+
+    def login(self, username=None, password=None):
+        """
+        Login to the Flickr webservice using the given credentials.
+
+        @param username: login of the user
+        @type username:  string
+        @param password: password of the user
+        @type password:  string
+        @rtype:          L{twisted.internet.defer.Deferred}
+        """
+        # yay for recursive imports
+        from elisa.plugins.flickr.flickr_api import authenticate_1, authenticate_2, \
+             _get_token_file
+
+        username = username or ''
+        password = password or ''
+
+        login_model = LoginModel()
+        login_model.username = username
+        login_model.password = password
+
+        def connected_cb(auth_token, model):
+            self.info("Authentication succeeded: %r", auth_token)
+            model.success = True
+            model.auth_token = auth_token
+
+            self.auth_token = auth_token
+
+            return model
+
+        def parse_login_web_form(url, username, password):
+            twill.go(url)
+            twill.formvalue(1, 'login', username)
+            twill.formvalue(1, 'passwd', password)
+            twill.submit()
+            twill.config('readonly_controls_writeable', 'True')
+            try:
+                twill.formvalue(3, 'done_auth', '1')
+                twill.submit()
+            except:
+                # already granted
+                pass
+
+        def web_form_parsed(result, frob, model):
+            dfr = authenticate_2(self, frob)
+            dfr.addCallback(connected_cb, model)
+            return dfr
+
+        def auth_open_url(state, model):
+            self.debug("state = %r", state)
+            if 'token' in state:
+                return connected_cb(state['token'], model)
+            else:
+                dfr = threads.deferToThread(parse_login_web_form, state['url'],
+                                            username, password)
+                dfr.addCallback(web_form_parsed, state['frob'], model)
+                return dfr
+
+        # remove the previous auth_token (from a previous login)
+        token_file = _get_token_file()
+        if self.auth_token and os.path.exists(token_file):
+            os.unlink(token_file)
+
+        dfr = authenticate_1(self)
+        dfr.addCallback(auth_open_url, login_model)
+        return dfr
+
+    def logout(self):
+        """
+        Remove any reference to the current Flickr authentication
+        token and remove the file storing the Flickr frob.
+        """
+        from elisa.plugins.flickr.flickr_api import _get_token_file
+
+        self.debug("Logging out. Auth token was: %s", self.auth_token)
+        self.auth_token = None
+        token_file = _get_token_file()
+        if os.path.exists(token_file):
+            os.unlink(token_file)
+            self.debug("Removed auth frob file: %s", token_file)
 
     def get(self, uri, context_model=None):
         """
         GET request to the Flickr servers.
 
-        The request URI can be of one of the following forms:
+        The request URI can be of one of the following forms::
 
             http://api.flickr.com/services/rest/?.*
             http://farm{farm_id}.static.flickr.com/.*\.jpg
@@ -108,6 +214,8 @@ class FlickrResourceProvider(ResourceProvider):
                               L{elisa.core.utils.defer.Deferred}
         """
         url = str(uri)
+        self.debug("GET %s", url)
+
         # Select the correct HTTP client to target
         match = self.api_re.match(url)
         if match is not None:
@@ -167,6 +275,70 @@ class FlickrResourceProvider(ResourceProvider):
                         photo_model.isfriend = bool(int(photo.attributes['isfriend'].nodeValue))
                         photo_model.isfamily = bool(int(photo.attributes['isfamily'].nodeValue))
                         model.photos.append(photo_model)
+                elif rsp.getElementsByTagName('photosets'):
+                    # The response contains a list of photosets
+                    model.photosets = []
+                    sets = rsp.getElementsByTagName('photosets')[0].getElementsByTagName('photoset')
+                    for photoset in sets:
+                        set_model = FlickrPhotoSetModel()
+                        set_model.flickr_id = photoset.attributes['id'].nodeValue
+                        set_model.primary = int(photoset.attributes['primary'].nodeValue)
+                        set_model.secret = photoset.attributes['secret'].nodeValue
+                        set_model.farm = int(photoset.attributes['farm'].nodeValue)
+                        set_model.server = int(photoset.attributes['server'].nodeValue)
+                        set_model.photosnb = int(photoset.attributes['photos'].nodeValue)
+
+                        title = photoset.getElementsByTagName('title')[0]
+                        set_model.title = title.childNodes[0].nodeValue
+
+                        description = photoset.getElementsByTagName('description')[0]
+                        if description.childNodes:
+                            set_model.description = description.childNodes[0].nodeValue
+                        else:
+                            set_model.description = ""
+
+                        model.photosets.append(set_model)
+                elif rsp.getElementsByTagName('photoset'):
+                    # The response contains a list of photos
+                    model.photos = []
+                    photolist = rsp.getElementsByTagName('photoset')[0].getElementsByTagName('photo')
+                    owner = "Foo"
+                    for photo in photolist:
+                        photo_model = FlickrPhotoModel()
+                        photo_model.flickr_id = photo.attributes['id'].nodeValue
+                        photo_model.owner = owner
+                        photo_model.secret = photo.attributes['secret'].nodeValue
+                        photo_model.farm = int(photo.attributes['farm'].nodeValue)
+                        photo_model.server = int(photo.attributes['server'].nodeValue)
+                        photo_model.title = photo.attributes['title'].nodeValue
+                        model.photos.append(photo_model)
+                elif rsp.getElementsByTagName('contacts'):
+                    # The response contains a list of contacts
+                    model.contacts = []
+                    contacts = rsp.getElementsByTagName('contacts')[0].getElementsByTagName('contact')
+                    for contact in contacts:
+                        contact_model = FlickrContactModel()
+                        contact_model.nsid = contact.attributes['nsid'].nodeValue
+                        contact_model.username = contact.attributes['username'].nodeValue
+                        contact_model.iconserver = int(contact.attributes['iconserver'].nodeValue)
+                        contact_model.realname = contact.attributes['realname'].nodeValue
+                        contact_model.is_friend = int(contact.attributes['friend'].nodeValue)
+                        contact_model.is_family = int(contact.attributes['family'].nodeValue)
+                        contact_model.is_ignored = int(contact.attributes['ignored'].nodeValue)
+                        model.contacts.append(contact_model)
+                elif rsp.getElementsByTagName('who'):
+                    # The response contains a list of user tags
+                    model.tags = []
+                    tags = rsp.getElementsByTagName('who')[0].getElementsByTagName('tag')
+                    for tag in tags:
+                        tag_model = FlickrTagModel()
+                        tag_model.label = tag.firstChild.nodeValue
+                        # the private attribute is specific to our models
+                        # we set it to True here because the response results from a call
+                        # of the API flickr.tags.getListUser (request the user's tags)
+                        tag_model.private = True
+                        model.tags.append(tag_model)
+
                 elif rsp.getElementsByTagName('photo'):
                     # The response contains details about one photo
                     model.photo = FlickrPhotoModel()
@@ -223,6 +395,8 @@ class FlickrResourceProvider(ResourceProvider):
                     if _large is None:
                         _large = _medium
                     model.sizes = ImageModel()
+                    # TODO: implement image rotation
+                    model.sizes.can_rotate = False
                     model.sizes.references = [MediaUri(_small),
                                               MediaUri(_medium),
                                               MediaUri(_large)]
@@ -327,7 +501,7 @@ class FlickrResourceProvider(ResourceProvider):
         """
         POST request to the Flickr servers.
 
-        The request URI can be of one of the following forms:
+        The request URI can be of one of the following forms::
 
             http://api.flickr.com/services/upload/
             http://api.flickr.com/services/replace/
