@@ -14,4 +14,8 @@ setup(name='elisa-plugin-lirc',
     namespace_packages=['elisa', 'elisa.plugins'],
     packages=packages,
     package_dir=package_dir,
-    cmdclass=cmdclass)
+    cmdclass=cmdclass,
+    entry_points="""
+    [elisa.core.components.input_provider]
+    LircInput = elisa.plugins.lirc.lirc_input:LircInput
+    """)
