@@ -50,7 +50,6 @@ def shoutcast_decorator(controller):
     link.controller_path = '/poblesec/shoutcast'
     link.label = 'Shoutcast'
     link.icon = 'elisa.plugins.shoutcast.logo'
-    controller.model.append(link)
     return defer.succeed(None)
 
 
@@ -93,6 +92,9 @@ class GenericShoutcastController(HierarchyController):
         self.actions.extend(self.make_actions())
         return dfr
 
+    def is_empty(self):
+        return super(GenericShoutcastController, self).is_empty() and self.station is None
+
     def make_actions(self):
         actions = []
 
@@ -117,22 +119,14 @@ class GenericShoutcastController(HierarchyController):
         return actions
 
     def node_clicked(self, widget, item):
-        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
-        selected_item = self.nodes._widgets[widget_index]
-
-        if selected_item != self._previous_clicked:
-            selected_item.activate(previous=self._previous_clicked)
-
-            self._previous_clicked = selected_item
-
-            if isinstance(item, Action):
-                item.run()
-            elif isinstance(item, ShoutcastGenreModel):
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                browser.history.append_controller(self.path, item.name, uri=item.uri)
-            elif isinstance(item, ShoutcastRadioStationModel):
-                browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
-                browser.history.append_controller(self.path, item.name, uri=None, station=item)
+        if isinstance(item, Action):
+            item.run()
+        elif isinstance(item, ShoutcastGenreModel):
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            browser.history.append_controller(self.path, item.name, uri=item.uri)
+        elif isinstance(item, ShoutcastRadioStationModel):
+            browser = self.frontend.retrieve_controllers('/poblesec/browser')[0]
+            browser.history.append_controller(self.path, item.name, uri=None, station=item)
 
     def play_station(self, station):
         resource, dfr = application.resource_manager.get(station.get_playable, station)
