@@ -21,6 +21,8 @@ setup(name='elisa-plugin-shoutcast',
                   'elisa.plugins.shoutcast.i18n': ['*/LC_MESSAGES/*.po', '*.pot'],
                  },
     entry_points="""\
+    [elisa.core.components.resource_provider]
+    ShoutcastResource = elisa.plugins.shoutcast.shoutcast_resource:ShoutcastResource
     """,
     controller_mappings=[('/poblesec/shoutcast',
                           'elisa.plugins.shoutcast.controller:ShoutcastListSwitcherController')],
