@@ -43,7 +43,7 @@ import string
 
 API_SERVER = 'gdata.youtube.com'
 VIDEO_SERVER = 'www.youtube.com'
-IMG_SERVER = 'img.youtube.com'
+IMG_SERVER = 'i.ytimg.com'
 
 class YoutubeResourceProvider(ResourceProvider):
 
@@ -80,11 +80,17 @@ class YoutubeResourceProvider(ResourceProvider):
 
     def clean(self):
         # Close all open HTTP connections
-        return defer.DeferredList([self._api_client.close(),
+        dfr = defer.DeferredList([self._api_client.close(),
                                    self._video_client.close(),
                                    self._img_client.close()],
                                   consumeErrors=True)
 
+        def parent_clean(result):
+            return super(YoutubeResourceProvider, self).clean()
+
+        dfr.addCallback(parent_clean)
+        return dfr
+
     def get(self, uri, context_model=None):
         """
         GET request to the Youtube servers.
