@@ -21,6 +21,7 @@ Unit tests for the Youtube resource provider.
 """
 
 from twisted.trial.unittest import TestCase
+from elisa.core.utils.misc import run_functional_tests_check
 from twisted.internet import defer
 
 from elisa.plugins.youtube.resource_provider import YoutubeResourceProvider, \
@@ -44,6 +45,7 @@ class TestYoutubeResourceProvider(TestCase):
     """
 
     def setUp(self):
+        run_functional_tests_check()
         def created(provider):
             self.provider = provider
             return provider.initialize()
